package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kommons.core.IStruct2i
import ru.dbotthepony.kommons.vector.Vector2i

interface ICellAccess {
	/**
	 * with wrap-around
	 */
	fun getCell(x: Int, y: Int): AbstractCell
	fun getCell(pos: IStruct2i) = getCell(pos.component1(), pos.component2())

	/**
	 * without wrap-around
	 */
	fun getCellDirect(x: Int, y: Int): AbstractCell
	fun getCellDirect(pos: IStruct2i) = getCellDirect(pos.component1(), pos.component2())

	/**
	 * whenever cell was set
	 */
	fun setCell(x: Int, y: Int, cell: AbstractCell): Boolean

	/**
	 * Возвращает псевдослучайное Long для заданной позиции
	 *
	 * Для использования в рендерах и прочих вещах, которым нужно стабильное число на основе своей позиции
	 */
	fun randomLongFor(x: Int, y: Int): Long {
		var long = x * 738548L + y * 2191293543L
		long = long xor 8339437585692L
		long = (long ushr 4) or (long shl 52)
		long *= 7848344324L
		long = (long ushr 12) or (long shl 44)
		return long
	}

	/**
	 * Возвращает псевдослучайное нормализированное Double для заданной позиции
	 *
	 * Для использования в рендерах и прочих вещах, которым нужно стабильное число на основе своей позиции
	 */
	fun randomDoubleFor(x: Int, y: Int): Double {
		return (randomLongFor(x, y) / 9.223372036854776E18) / 2.0 + 0.5
	}

	fun randomLongFor(pos: Vector2i) = randomLongFor(pos.x, pos.y)
	fun randomDoubleFor(pos: Vector2i) = randomDoubleFor(pos.x, pos.y)
}

